#!/usr/bin/env python
# -*- coding: utf-8 -*-

# camwatchdog.py - monitor a logfile for CAM failure messages and invoke SVDRP commands in turn
# Licensed under the terms of the GNU GPLv3 or later
# Proprietary Software Must Perish!

# AUTHOR: Johannes Truschnigg ( johannes@truschnigg.info || http://johannes.truschnigg.info )

import socket
import re
import sys
import os
import time
import signal


def svdrp_send(host, commands, sleeptime=0.5, port=6419):
    vdr_s = socket.create_connection((host, port))
    vdr = vdr_s.makefile("r+b")
    greeting = vdr.readline()
    print "VDR greeting:", greeting
    for c in commands:
        lastline = False
        print "Sending via SVDRP:", c
        resp = ""
        line = ""
        vdr.write(c + "\r\n")
        vdr.flush()
        while lastline == False:
            line = vdr.readline()
            if line.count("-", 0, 4) == 0:
                lastline = True
            resp += line
        print "VDR response:", resp
        time.sleep(sleeptime)
    vdr.close()
    return True

def main():
    msg_fail = re.compile(".*dvb_ca adapter [0-9]+: DVB CAM link initialisation failed :.")
    try:
        filename = sys.argv[1]
    except IndexError:
        print >> sys.stderr, "%s: Need a logfile path to watch on argv[1]." % os.path.basename(sys.argv[0])
        sys.exit(100)
    try:
        fp = open(filename, "r", 0)
    except IOError:
        print >> sys.stderr, "%s: Need an existing, readable logfile to watch on argv[1]." % os.path.basename(sys.argv[0])
        sys.exit(100)
    if "CAMWATCHD_SKIPCONTENT" in os.environ:
        fp.seek(0, os.SEEK_END) #we're only interested in lines appearing in the logfile AFTER this run has initiated
    prev_offset = fp.tell()
    offset = prev_offset
    print "Monitoring file '%s' initial offset: %d" % (filename, offset)
    while True:
        sys.stdout.flush()
        prev_offset = offset
        offset = fp.tell()
        if prev_offset != offset:
            pass
        line = fp.readline().strip()
        if not line: #no (complete) line could be read, or there was no new file content at all
            time.sleep(1) #sleep for a while to avoit busy looping
            fp.seek(offset) #rewind file descriptor to the previous position
        else:
            print "New line in log:", line
            if msg_fail.match(line):
                print "Watchdog triggered by:", line
                svdrp_send("127.0.0.1", ("HITK Menu", "HITK 1 1", "HITK 5", "HITK Green", "HITK Ok", "HITK Menu"))

if __name__ == "__main__":
    main()
